Attribute VB_Name = "modWakeUp"
' Dieser Source stammt von http://www.activevb.de
' und kann frei verwendet werden. Fr eventuelle Schden
' wird nicht gehaftet.
'
' Um Fehler oder Fragen zu klren, nutzen Sie bitte unser Forum.
' Ansonsten viel Spa und Erfolg mit diesem Source!


Option Explicit

' ----==== GetVersionEx Const ====----
Private Const VER_PLATFORM_WIN32_NT As Long = 2&

' ----==== SetThreadExecutionState Const ====----
Private Const ES_CONTINUOUS As Long = &H80000000
Private Const ES_DISPLAY_REQUIRED As Long = &H2
Private Const ES_SYSTEM_REQUIRED As Long = &H1

' ----==== GoSleep Enum ====----
Public Enum GoSleepState
    SystemResume = 0
    PwrSuspendNotAllowed = 1
    PwrHibernateNotAllowed = 2
    WaitableTimerNotCreated = 3
    WaitableTimerNotSet = 4
    SystemNotSuspend = 5
    SystemNotHibernate = 6
    MonitorCantActivate = 7
End Enum

' ----==== GetVersionEx Type ====----
Private Type OSVERSIONINFO
    dwOSVersionInfoSize As Long
    dwMajorVersion As Long
    dwMinorVersion As Long
    dwBuildNumber As Long
    dwPlatformId As Long
    szCSDVersion As String * 128
End Type

' ----==== SetWaitableTimer Type ====----
Private Type FILETIME
    lngLowDateTime As Long
    lngHighDateTime As Long
End Type

' ----==== Kernel32 API Declarations ====----
Private Declare Function CancelWaitableTimer Lib "Kernel32" ( _
                         ByVal hTimer As Long) As Long
                         
Private Declare Function CloseHandle Lib "Kernel32" ( _
                         ByVal hObject As Long) As Long
                         
Private Declare Function CreateWaitableTimer Lib "Kernel32" _
                         Alias "CreateWaitableTimerA" ( _
                         ByVal lpTimerAttributes As Long, _
                         ByVal bManualReset As Long, _
                         ByVal lpTimerName As String) As Long
                         
Private Declare Function GetVersionEx Lib "Kernel32" _
                         Alias "GetVersionExA" ( _
                         ByRef lpVersionInformation As OSVERSIONINFO) As Long
                         
Private Declare Function SetSystemPowerState Lib "Kernel32" ( _
                         ByVal fSuspend As Long, _
                         ByVal bForce As Boolean) As Long
                         
Private Declare Function SetThreadExecutionState Lib "Kernel32" ( _
                         ByVal esFlags As Long) As Long
                         
Private Declare Function SetWaitableTimer Lib "Kernel32" ( _
                         ByVal hTimer As Long, _
                         ByRef lpDueTime As FILETIME, _
                         ByVal lPeriod As Long, _
                         ByVal pfnCompletionRoutine As Long, _
                         ByVal lpArgToCompletionRoutine As Long, _
                         ByVal fResume As Long) As Long
                         
' ----==== Powrprof API Declarations ====----
Private Declare Function IsPwrHibernateAllowed Lib "Powrprof" () As Long

Private Declare Function IsPwrSuspendAllowed Lib "Powrprof" () As Long

Private Declare Function SetSuspendState Lib "Powrprof" ( _
                         ByVal Hibernate As Long, _
                         ByVal ForceCritical As Long, _
                         ByVal DisableWakeEvent As Long) As Long
                         
' ------------------------------------------------------
' Funktion     : EnableMonitorAfterSleep
' Beschreibung : Einschalten des Monitors nach
'                Suspend- oder Hibernate-Modus
' bergabewert : True = Monitor ein / False = Monitor aus
' Rckgabewert : Erfolgreich = previous thread execution state
'                Fehlgeschlagen = null
' ------------------------------------------------------
Private Function EnableMonitorAfterSleep(ByVal EnableMonitor As Boolean) _
    As Long
    
    Dim lngFlags As Long
    
    lngFlags = ES_SYSTEM_REQUIRED Or ES_CONTINUOUS
    
    ' ist EnableMonitor = True dann
    If EnableMonitor Then
    
        ' Monitor nach Standby oder Ruhezustand wieder einschalten
        lngFlags = lngFlags Or ES_DISPLAY_REQUIRED
        
    End If
    
    EnableMonitorAfterSleep = SetThreadExecutionState(lngFlags)
    
End Function

' ------------------------------------------------------
' Funktion     : GoSleep
' Beschreibung : Schaltet das System in den Suspend- oder
'                Hibernate-Modus und weckt diesen nach einer
'                bestimmten Zeitspanne wieder auf (sofern das
'                vom System untersttzt und aktiviert ist)
' bergabewert : Sec = Zeit in Sekunden bis zum aufwecken
'                Suspend: True = Suspend-Mode / False = Hibernate-Mode
'                SetSuspendIfHibernateNotAllowed:
'                      True = probiere den Suspend-Mode wenn
'                             Hibernate-Mode nicht funktioniert
'                      False = Suspend-Mode nicht probieren falls
'                              Hibernate-Mode nicht funktioniert
'                ForceCritical: False = sanftes / True = hartes
'                               herrunterfahren des Systems
'                EnableMonitor: True = Monitor nach aufwecken einschalten
'                               False = Monitor nach aufwecken nicht
'                                       einschalten
' Rckgabewert : GoSleepState = Enum GoSleepState
' ------------------------------------------------------
Public Function GoSleep(ByVal Sec As Long, Optional ByVal Suspend As _
    Boolean = True, Optional ByVal SetSuspendIfHibernateNotAllowed As _
    Boolean = True, Optional ByVal ForceCritical As Boolean = False, _
    Optional ByVal EnableMonitor As Boolean = True) As GoSleepState
    
    Dim dblDelay As Double
    Dim dblDelayLow As Double
    Dim dblUnits As Double
    Dim tFILETIME As FILETIME
    Dim hTimer As Long
    
    ' ---=== Suspend-Mode ===---
    If Suspend Then
    
        ' wenn der Suspend-Mode nicht untersttzt wird, dann
        If Not CBool(IsPwrSuspendAllowed) Then
        
            ' Rckgabewert setzen
            GoSleep = PwrSuspendNotAllowed
            
            ' Funktion verlassen
            Exit Function
        
        End If
        
    Else
    
        ' ---=== Hibernate-Mode ===---
        
        ' wenn der Hibernate-Mode nicht untersttzt wird, dann
        If Not CBool(IsPwrHibernateAllowed) Then
        
            ' Suspend-Mode testen wenn der Hibernate-Mode nicht
            ' untersttzt wird
            If SetSuspendIfHibernateNotAllowed Then
            
                ' wenn der Suspend-Mode nicht untersttzt wird, dann
                If Not CBool(IsPwrSuspendAllowed) Then
                
                    ' Rckgabewert setzen
                    GoSleep = PwrSuspendNotAllowed
                    
                    ' Funktion verlassen
                    Exit Function
                
                Else
                
                    ' Suspend-Mode verwenden
                    Suspend = True
                
                End If
                
            Else
            
                ' Rckgabewert setzen
                GoSleep = PwrHibernateNotAllowed
                
                ' Funktion verlassen
                Exit Function
                
            End If
        End If
    End If
    
    ' WaitableTimer-Objekt erstellen
    hTimer = CreateWaitableTimer(0&, True, vbNullChar)
    
    If hTimer = 0 Then
    
        ' Rckgabewert setzen
        GoSleep = WaitableTimerNotCreated
        
        ' Funktion verlassen
        Exit Function
        
    Else
    
        dblUnits = CDbl(&H10000) * CDbl(&H10000)
        dblDelay = CDbl(Sec) * 1000 * 10000
        tFILETIME.lngHighDateTime = -CLng(dblDelay / dblUnits) - 1
        
        dblDelayLow = -dblUnits * (dblDelay / dblUnits - Fix(dblDelay / _
            dblUnits))
            
        If dblDelayLow < CDbl(&H80000000) Then
        
            dblDelayLow = dblUnits + dblDelayLow
            tFILETIME.lngHighDateTime = tFILETIME.lngHighDateTime + 1
            
        End If
        
        tFILETIME.lngLowDateTime = CLng(dblDelayLow)
        
        ' Falls das starten des WaitableTimer fehlschlgt
        If SetWaitableTimer(hTimer, tFILETIME, 0&, 0&, 0&, True) = 0 Then
        
            ' WaitableTimer auf inaktiv setzen
            Call CancelWaitableTimer(hTimer)
            
            ' WaitableTimer-Handle schlieen
            Call CloseHandle(hTimer)
            
            ' Rckgabewert setzen
            GoSleep = WaitableTimerNotSet
            
            ' Funktion verlassen
            Exit Function
            
        Else
            
            DoEvents
        
            ' ---=== Suspend-Mode ===---
            If Suspend Then
            
                ' OS >= WinNT
                If IsWinNT Then
                
                    ' Suspend-Mode
                    If SetSuspendState(False, ForceCritical, False) = 0 _
                        Then
                        
                        ' Rckgabewert setzen
                        GoSleep = SystemNotSuspend
                        
                    Else
                    
                        ' Falls das einschalten des Monitors fehlschlgt
                        If EnableMonitorAfterSleep(EnableMonitor) = 0 _
                            Then GoSleep = MonitorCantActivate
                            
                    End If
                    
                Else
                
                    ' OS <= WinMe
                    
                    ' Suspend-Mode
                    If SetSystemPowerState(True, ForceCritical) = 0 Then
                    
                        ' Rckgabewert setzen
                        GoSleep = SystemNotSuspend
                        
                    Else
                    
                        ' Falls das einschalten des Monitors fehlschlgt
                        If EnableMonitorAfterSleep(EnableMonitor) = 0 _
                            Then GoSleep = MonitorCantActivate
                            
                    End If
                    
                End If
                
            Else
            
                ' ---=== Hibernate-Mode ===---
                
                ' OS >= WinNT
                If IsWinNT Then
                
                    ' Hibernate-Mode
                    If SetSuspendState(True, ForceCritical, False) = 0 Then
                    
                        ' Rckgabewert setzen
                        GoSleep = SystemNotHibernate
                        
                    Else
                    
                        ' Falls das einschalten des Monitors fehlschlgt
                        If EnableMonitorAfterSleep(EnableMonitor) = 0 _
                            Then GoSleep = MonitorCantActivate
                            
                    End If
                    
                Else
                
                    ' OS <= WinMe
                    
                    ' Hibernate-Mode
                    If SetSystemPowerState(False, ForceCritical) = 0 Then
                    
                        ' Rckgabewert setzen
                        GoSleep = SystemNotHibernate
                        
                    Else
                    
                        ' Falls das einschalten des Monitors fehlschlgt
                        If EnableMonitorAfterSleep(EnableMonitor) = 0 _
                            Then GoSleep = MonitorCantActivate
                            
                    End If
                    
                End If
                
            End If
            
            ' WaitableTimer auf inaktiv setzen
            Call CancelWaitableTimer(hTimer)
            
            ' WaitableTimer-Handle schlieen
            Call CloseHandle(hTimer)
            
            ' Rckgabewert setzen
            GoSleep = SystemResume
            
        End If
        
    End If
    
End Function

' ------------------------------------------------------
' Funktion     : IsWinNT
' Beschreibung : OS-System ermitteln
' bergabewert : keiner
' Rckgabewert : True = NT-Based System / False DOS-Based System
' ------------------------------------------------------
Private Function IsWinNT() As Boolean

    Dim tOSVERSIONINFO As OSVERSIONINFO
    
    tOSVERSIONINFO.dwOSVersionInfoSize = Len(tOSVERSIONINFO)
    
    Call GetVersionEx(tOSVERSIONINFO)
    
    IsWinNT = (tOSVERSIONINFO.dwPlatformId = VER_PLATFORM_WIN32_NT)
    
End Function


